import React from 'react';
import {View, Text, StyleSheet, TouchableOpacity} from 'react-native';
import {Chip} from 'react-native-digitcamera/src/Screens/AutoPI/common/Chip';
import EntypoIcon from 'react-native-vector-icons/Entypo';
const Rejected = () => (
  <Chip
    background={'#EE000014'}
    text={'Rejected'}
    textColor={'#EE0000'}
    iconColor={'#EE0000'}
    iconName={'close'}
  />
);
const Approved = () => (
  <Chip
    background={'#00AA5514'}
    text={'Approved'}
    textColor={'#00AA55'}
    iconColor={'#00AA55'}
    iconName={'check'}
  />
);

const Uploaded = () => (
  <Chip
    background={'#00AA5514'}
    text={'Uploaded'}
    textColor={'#00AA55'}
    iconColor={'#00AA55'}
    iconName={'check'}
  />
);

const AutoApproved = () => (
  <Chip
    background={'#00AA5514'}
    text={'Auto Approved'}
    textColor={'#00AA55'}
    iconColor={'#00AA55'}
    iconName={'check'}
  />
);
const Pending = () => (
  <Chip
    background={'#FFBB0030'}
    text={'Pending'}
    textColor={'#FFAA00'}
    iconColor={'#FFAA00'}
    iconName={'clock-check-outline'}
  />
);

const InProgress = () => (
  <Chip
    background={'#FFBB0030'}
    text={'PI in Progress'}
    textColor={'#FFAA00'}
    iconColor={'#FFAA00'}
    iconName={'clock-check-outline'}
  />
);

const styles = StyleSheet.create({
  activityMain: {
    backgroundColor: '#FFFFFF',
    elevation: 1,
    borderRadius: 8,
    margin: 8,
  },
  autoApprovedView: {
    backgroundColor: '#00AA55',
    borderTopLeftRadius: 8,
    borderTopRightRadius: 8,
    paddingVertical: 8,
    flexDirection: 'row',
    justifyContent: 'center',
    alignItems: 'center',
  },
  autoApprovedText: {
    fontFamily: 'Muli-Medium',
    fontWeight: '700',
    fontSize: 12,
    lineHeight: 15,
    color: '#FFFFFF',
    marginLeft: 8,
  },
  activityDetailsView: {padding: 16},
  activityTitleView: {
    marginBottom: 8,
    flexDirection: 'row',
    alignItems: 'center',
  },
  activityTitle: {
    fontFamily: 'Muli-Bold',
    fontWeight: '600',
    fontSize: 14,
    lineHeight: 18,
    color: '#444444',
  },
  activityStatus: {
    marginBottom: 8,
    fontFamily: 'Muli-Regular',
    fontStyle: 'italic',
    fontWeight: '400',
    fontSize: 11,
    lineHeight: 14,
    color: '#999999',
  },
  rejectedView: {
    marginTop: 8,
    backgroundColor: '#F5F5F5',
    padding: 12,
    borderRadius: 8,
  },
  reinspectionView: {
    marginTop: 8,
    backgroundColor: '#FFBB001F',
    padding: 12,
    borderRadius: 8,
  },
  reinspectionButton: {
    marginTop: 8,
    backgroundColor: '#FFBB00',
    alignSelf: 'flex-start',
    borderRadius: 4,
    padding: 6,
    elevation: 2,
  },
  reinspectionTitle: {
    fontFamily: 'Muli-Bold',
    fontWeight: '400',
    fontSize: 11,
    lineHeight: 14,
  },
  reasonText: {
    fontFamily: 'Muli-Regular',
    fontWeight: '400',
    fontSize: 11,
    lineHeight: 14,
    color: '#999999',
  },
});

const RejectedView = ({reason}) => {
  return (
    <View style={styles.rejectedView}>
      <Text style={styles.reasonText}>{reason}</Text>
    </View>
  );
};

const ReInspectionView = ({reason, onPress}) => {
  return (
    <View style={styles.reinspectionView}>
      <Text style={styles.reasonText}>{reason}</Text>
      <TouchableOpacity
        onPress={onPress}
        activeOpacity={0.8}
        style={styles.reinspectionButton}
      >
        <Text style={styles.reinspectionTitle}>Start Reinspection</Text>
      </TouchableOpacity>
    </View>
  );
};

export const PI_STATUS = {
  Uploaded: 'Uploaded',
  'SelfSurvey-Approved': 'SelfSurvey-Approved',
  'SelfSurvey-Rejected': 'SelfSurvey-Rejected',
  'SelfSurvey-Reinspection Requested': 'SelfSurvey-Reinspection Requested',
  'PI-Pending': 'PI-Pending',
  'PI in progress': 'PI in progress',
};

const getStatusChip = (piStatus, isAutoApproved) => {
  if (
    piStatus === PI_STATUS['SelfSurvey-Approved'] &&
    isAutoApproved === true
  ) {
    return <AutoApproved />;
  }
  switch (piStatus) {
    case PI_STATUS['SelfSurvey-Approved']:
      return <Approved />;
    case PI_STATUS['SelfSurvey-Rejected']:
      return <Rejected />;
    case PI_STATUS['SelfSurvey-Reinspection Requested']:
      return <Rejected />;
    case PI_STATUS['PI-Pending']:
      return <Pending />;
    case PI_STATUS['PI in progress']:
      return <InProgress />;
    case PI_STATUS['Uploaded']:
      return <Uploaded />;
  }
};

const getReasonView = (piStatus, reason) => {
  if (piStatus === PI_STATUS['SelfSurvey-Rejected']) {
    return <RejectedView reason={reason} />;
  }
  if (piStatus === PI_STATUS['SelfSurvey-Reinspection Requested']) {
    return <ReInspectionView reason={reason} />;
  }
  return null;
};

const ActivityCard = ({
  regNumber,
  model,
  date,
  piStatus,
  reason,
  isAutoApproved,
  resultTime,
}) => {
  return (
    <View style={styles.activityMain}>
      {piStatus === PI_STATUS['SelfSurvey-Approved'] &&
        isAutoApproved === true && (
          <View style={styles.autoApprovedView}>
            <EntypoIcon name="flash" size={16} color={'#FFFFFF'} />
            <Text style={styles.autoApprovedText}>Auto Approved in 2 min</Text>
          </View>
        )}
      <View style={styles.activityDetailsView}>
        <View>
          <View style={styles.activityTitleView}>
            <Text style={styles.activityTitle}>{regNumber}</Text>
            <Text
              style={[styles.activityTitle, {flex: 1, textAlign: 'left'}]}
              numberOfLines={1}
            >
              {model ? ` - ${model}` : ''}
            </Text>
          </View>
          <Text style={styles.activityStatus}>
            {resultTime ? resultTime : date}
          </Text>
        </View>
        <View>{getStatusChip(piStatus, isAutoApproved)}</View>
        {getReasonView(piStatus, reason)}
      </View>
    </View>
  );
};

export default ActivityCard;
