import {View, Text, StyleSheet, FlatList} from 'react-native';
import React, {useEffect, useReducer} from 'react';
import {ActivityIndicator} from 'react-native-paper';
import style from 'react-native-digitcamera/src/common/CommonStyle';
import Database from '../../../../../../Components/common/DAsyncStorage';
import ActivityCard, {
  PI_STATUS,
} from 'react-native-digitcamera/src/Screens/AutoPI/common/ActivityCard';
import {canGoVideoPi} from 'react-native-digitcamera/src/common/Constants';
import store from 'react-native-digitcamera/src/redux/store';
import {useSelector} from 'react-redux';
import {
  addCompletedActivity,
  addPICompletedActivity,
  updateCompletedActivity,
} from 'react-native-digitcamera/src/redux/reducers/autopiReducer';
import {findAndDeleteOlderActivitiesAsync} from 'react-native-digitcamera/src/Screens/SilentUploadV2/DatabaseHelpers';
import {getActivityDetails} from 'react-native-digitcamera/src/Screens/AutoPI/services/getActivityDetails';
import moment from 'moment';

const _emptyItem = () => {
  return (
    <View style={{flex: 1, justifyContent: 'center', alignItems: 'center'}}>
      <Text style={[style.muliRegLG, {fontSize: 14}]}>
        No completed data available
      </Text>
    </View>
  );
};

const _renderItem = ({item, index}) => {
  return <ActivityCard {...item} />;
};

const getFormatedTime = time => {
  return moment(time).format('DD MMM YYYY, h:mm:ss a');
};

const modifyResult = result => {
  /*
    response sample 
    {
     "activityId": 1426197,
     "registrationNumber": "DL1LW6002",
     "isAutoApproved": false,
     "statusName": "SelfSurvey-Approved",
     "policyNumber": "D300379868",
     "approvedTime": "2023-03-01 13:24:25.0",
     "model": "TATA ACE",
     "turnAroundTime": "00:08:31",
     "status": 45
}  {
     "activityId": 1422801,
     "registrationNumber": "GJ36L2980",
     "pendingReason": "Front windshield not clear, Required clear front Windshield image ",
     "statusName": "PI-Pending",
     "model": "MARUTI SUZUKI MARUTI CELERIO ZXI",
     "turnAroundTime": "47:15:32",
     "status": 65,
     "pendingTime": "2023-03-02 11:37:12.0"
}  {
     "activityId": 1421189,
     "rejectedTime": "2023-02-27 17:37:07.0",
     "registrationNumber": "OD01AF9691",
     "statusName": "SelfSurvey-Rejected",
     "policyNumber": "D300377956",
     "model": "MARUTI SUZUKI BALENO ALPHA PETROL",
     "rejectionReason": "damage beyond acceptance limit",
     "turnAroundTime": "02:54:18",
     "status": 46
}  {
     "activityId": 1421511,
     "reInspectionReason": "Test Case,Invalid Video,Poor Lighting - video not taken in the proper lighting",
     "registrationNumber": "MH50L6169",
     "statusName": "SelfSurvey-Reinspection Requested",
     "model": "HYUNDAI GRANDI10NIOS1.2MTKAPPASPORTZ",
     "turnAroundTime": "55:24:34",
     "reinspectionedTime": "2023-03-02 00:18:02.0",
     "status": 50
}  {
     "activityId": 1422795,
     "registrationNumber": "GJ12DS3213",
     "docReceivedTime": "2023-02-28 12:45:06.0",
     "statusName": "PI in progress",
     "model": "MORRIS GARAGES HECTOR PE 1 5 DCT SHARP",
     "turnAroundTime": "00:26:35",
     "status": 5
}
  
  */
  switch (result.statusName) {
    case PI_STATUS['SelfSurvey-Rejected']:
      return {
        activityId: result.activityId,
        piStatus: result.statusName,
        resultTime: getFormatedTime(result.rejectedTime),
        reason: result.rejectionReason,
        turnAroundTime: result.turnAroundTime,
        status: result.status,
        model: result.model,
      };
    case PI_STATUS['SelfSurvey-Approved']: {
      if (result?.isAutoApproved) {
        return {
          activityId: result.activityId,
          piStatus: result.statusName,
          resultTime: getFormatedTime(result.rejectedTime),
          isAutoApproved: result?.isAutoApproved,
          reason: '',
          turnAroundTime: result.turnAroundTime,
          status: result.status,
          model: result.model,
        };
      } else {
        return {
          activityId: result.activityId,
          piStatus: result.statusName,
          resultTime: getFormatedTime(result.approvedTime),
          reason: '',
          turnAroundTime: result.turnAroundTime,
          status: result.status,
          model: result.model,
        };
      }
    }
    case PI_STATUS['SelfSurvey-Reinspection Requested']:
      return {
        activityId: result.activityId,
        piStatus: result.statusName,
        resultTime: getFormatedTime(result.reinspectionedTime),
        reason: result.reInspectionReason,
        turnAroundTime: result.turnAroundTime,
        status: result.status,
        model: result.model,
      };
    case PI_STATUS['PI-Pending']:
      return {
        activityId: result.activityId,
        piStatus: result.statusName,
        resultTime: getFormatedTime(result.pendingTime),
        reason: result.pendingReason,
        turnAroundTime: result.turnAroundTime,
        status: result.status,
        model: result.model,
      };
    case PI_STATUS['PI in progress']:
      return {
        activityId: result.activityId,
        piStatus: result.statusName,
        reason: '',
        turnAroundTime: result.turnAroundTime,
        status: result.status,
        model: result.model,
      };
  }
};

const checkActivityDetailsAsync = async activityId => {
  /*
  {
    "activityId": 1421189,
    "registrationNumber": "OD01AF9691",
    "statusName": "SelfSurvey-Rejected",
    "policyNumber": "D300377956",
    "RejectedTime": "2023-02-27 17:37:07.0",
    "rejectionReason": "damage beyond acceptance limit",
    "turnAroundTime": "02:54:18",
    "status": 46
     model: "Maruti Suzuki",
}
  */
  const [result, error] = await getActivityDetails(activityId);
  if (result) {
    return modifyResult(result);
  }
};

const hydrateActivitiesAsync = async () => {
  const mPIMediaStatus = await Database.get('PIMediaStatus');

  //get only private car activities

  if (mPIMediaStatus instanceof Array) {
    const piPendingActivities = [];
    const piCompletedActivities = [];

    mPIMediaStatus.forEach(item => {
      //pi not completed activities
      if (item.activityStatus === 'COMPLETED') {
        const status = item.piStatus;
        if (PI_STATUS[status]) {
          store.dispatch(addPICompletedActivity(item));
        } else {
          store.dispatch(addCompletedActivity(item));
        }
      }
    });
  }
};

const updateMediaStatus = async result => {
  const mPIMediaStatus = await Database.get('PIMediaStatus');
  if (mPIMediaStatus instanceof Array) {
    const mappedActivities = mPIMediaStatus.map(msItem => {
      if (msItem.activityId === result.activityId) {
        const {
          piStatus,
          resultTime,
          reason,
          turnAroundTime,
          status,
          model,
        } = result;
        if (result?.isAutoApproved) {
          return {
            ...msItem,
            piStatus,
            resultTime,
            reason,
            turnAroundTime,
            status,
            model,
            isAutoApproved: result?.isAutoApproved,
          };
        }
        return {
          ...msItem,
          piStatus,
          resultTime,
          reason,
          turnAroundTime,
          status,
          model,
        };
      }
      return msItem;
    });
    await Database.set('PIMediaStatus', mappedActivities);
  }
};

const CompletedSceneView = () => {
  const {activities} = useSelector(state => state.autoPI);
  useEffect(() => {
    hydrateActivitiesAsync();
    const checkStatusAsync = async item => {
      const activityId = item.activityId;
      const result = await checkActivityDetailsAsync(activityId);
      if (result) {
        store.dispatch(updateCompletedActivity(result));
        await updateMediaStatus(result);
      }
    };

    //get pi status for each activity
    const getPIStatus = () => {
      //get activities from redux
      //initialized to piStatus:'pending' & reason:''
      const {activities} = store.getState().autoPI;

      if (activities instanceof Array) {
        for (let aIdx = 0; aIdx < activities.length; aIdx++) {
          //for each pending activity run a check pi status
          //after every 2 sec
          //because of the async storage slowness
          const item = activities[aIdx];
          if (
            item.piStatus === PI_STATUS['Uploaded'] ||
            item.piStatus === PI_STATUS['PI in progress'] ||
            item.piStatus === PI_STATUS['PI-Pending']
          ) {
            setTimeout(() => {
              checkStatusAsync(item);
            }, (aIdx + 1) * 2000);
          }
        }
      }
    };

    /*
    call only once component is mounted
    execution is delayed by 3 seconds so that PIMediaStatus is fully loaded into redux
    calling this immediately would not work as loading data into redux from database will take time
    */
    setTimeout(() => {
      getPIStatus();
    }, 3000);

    //then call every 2 min
    const itr = setInterval(() => {
      getPIStatus();
    }, 10000);

    return () => {
      if (itr) {
        clearInterval(itr);
      }
    };
  }, []);

  return (
    <View style={styles.main}>
      <FlatList
        data={activities}
        keyExtractor={(item, index) => item.activityId}
        ListEmptyComponent={_emptyItem}
        renderItem={_renderItem}
        contentContainerStyle={{flexGrow: 1, backgroundColor: '#F5F5F5'}}
      />
    </View>
  );
};

export default CompletedSceneView;

const styles = StyleSheet.create({
  main: {
    flex: 1,
    backgroundColor: '#FFFFFF',
  },
});
